# qenerate: plugin=pydantic_v2

query AssignmentGroupIndex($course_id: ID!) {
    legacyNode(_id: $course_id, type: Course) {
        ... on Course {
            assignmentGroups {
                id
                legacy_id: _id
                name
                groupWeight
            }
        }
    }
}