# qenerate: plugin=pydantic_v2

query CourseIndex {
    allCourses {
        _id
        name
        courseNickname
        settings {
            # This value isn't actually used, but something needs to be included to determine if the settings object is null
            restrictStudentPastView
        }
        term {
            id
            startAt
            endAt
            name
        }
    }
}