# qenerate: plugin=pydantic_v2

query ModuleIndex($course_id: ID!) {
    legacyNode(_id: $course_id, type: Course) {
        ... on Course {
            modulesConnection {
                nodes {
                    name
                    id
                    moduleItems {
                        content {
                            ... on File {
                                _id
                                displayName
                                createdAt
                                updatedAt
                                url
                                size
                                mimeClass
                                contentType
                            }
                            ... on Page {
                                _id
                                title
                                updatedAt
                                createdAt
                            }
                        }
                    }
                }
            }
        }
    }
}